<?php
namespace Indeed\Ihc\PaymentGateways;
/*
@since 7.4
*/


class PayIr extends \Indeed\Ihc\PaymentGateways\PaymentAbstract
{

  protected $attributes       = array();
  protected $redirectUrl      = '';
  protected $abort            = false;
  protected $paymentTypeLabel = 'PayIr Payment';
  protected $currency         = '';
 
    public function __construct()
    {
        $this->currency = get_option('ihc_currency');
        include_once IHC_PATH . 'classes/PaymentGateways/PayIr/lib/nusoap.php';
    }

    public function send_payir($api, $amount, $redirect, $mobile = null, $factorNumber = null, $description = null) {
	  return curl_post_payir('https://pay.ir/pg/send', [
		'api'          => $api,
		'amount'       => $amount,
		'redirect'     => $redirect,
		'mobile'       => $mobile,
		'factorNumber' => $factorNumber,
		'description'  => $description,
	  ]);
    }

    public function verify_payir($api, $token) {
	   return curl_post_payir('https://pay.ir/pg/verify', [
		 'api' 	=> $api,
		 'token' => $token,
	   ]);
    }

    public function curl_post_payir($url, $params){
	   $ch = curl_init();
	   curl_setopt($ch, CURLOPT_URL, $url);
	   curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
	   curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
	   curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	   curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	   curl_setopt($ch, CURLOPT_HTTPHEADER, [
	 	'Content-Type: application/json',
	   ]);
	   $res = curl_exec($ch);
	   curl_close($ch);

	   return $res;
    }
    
	
  	public function doPayment(){
      \Ihc_User_Logs::set_user_id(@$this->attributes['uid']);
      \Ihc_User_Logs::set_level_id(@$this->attributes['lid']);
      \Ihc_User_Logs::write_log( __('PayIr Payment: Start process', 'ihc'), 'payments');
      //$client = new SoapClient('https://www.PayIr.com/pg/services/WebGate/wsdl', ['encoding' => 'UTF-8']);
      $PayIr_email = get_option('ihc_PayIr_email');
      $levels = get_option('ihc_levels');
      $sandbox = get_option('ihc_PayIr_sandbox');
      $r_url = get_option('ihc_PayIr_return_page');

      if(!$r_url || $r_url==-1){
        $r_url = get_option('page_on_front');
      }
      $r_url = get_permalink($r_url);
      if (!$r_url){
        $r_url = get_home_url();
      }
      
      $err = false;
      if (isset($levels[$this->attributes['lid']])){
        $level_arr = $levels[$this->attributes['lid']];
        if ($level_arr['payment_type']=='free' || $level_arr['price']=='') $err = true;
      } else {
        \Ihc_User_Logs::write_log( __('PayIr Payment: Level is free, no payment required.', 'ihc'), 'payments');
        $err = true;
      }
      // USER ID
      if (!empty($this->attributes['uid'])){
        $uid = $this->attributes['uid'];
      } else {
        $uid = get_current_user_id();
      }
      if (!$uid){
        $err = true;
        \Ihc_User_Logs::write_log( __('PayIr Payment: Error, user id not set.', 'ihc'), 'payments');
      } else {
        \Ihc_User_Logs::write_log( __('PayIr Payment: set user id @ ', 'ihc') . $uid, 'payments');
      }

      /// dynamic price
      $level_arr['price'] = $this->applyDynamicPrice($level_arr['price']);

      if ($err){
        ////if level it's not available for some reason, go back to prev page
        \Ihc_User_Logs::write_log( __('PayIr Payment: stop process, redirect home.', 'ihc'), 'payments');
        //header( 'location:'. $r_url );
        //exit();
      } else {
        $custom_data = json_encode(array('user_id' => $uid, 'level_id' => $this->attributes['lid'] ));
      }

      $site_url = site_url();
      $site_url = trailingslashit($site_url);
      $notify_url = add_query_arg('ihc_action', 'PayIr', $site_url);

      \Ihc_User_Logs::write_log( __('PayIr Payment: set ipn url @ ', 'ihc') . $notify_url, 'payments');

      $reccurrence = FALSE;
      if (isset($level_arr['access_type']) && $level_arr['access_type']=='regular_period'){
        $reccurrence = TRUE;
        \Ihc_User_Logs::write_log( __('PayIr Payment: Recurrence payment set.', 'ihc'), 'payments');
      }

      //coupons
      $coupon_data = array();
      if (!empty($this->attributes['ihc_coupon'])){
        $coupon_data = ihc_check_coupon($this->attributes['ihc_coupon'], $this->attributes['lid'] );
        \Ihc_User_Logs::write_log( __('PayPal Payment: the user used the following coupon: ', 'ihc') . $this->attributes['ihc_coupon'], 'payments');
      }
      
      update_option('Status_zal','lll');
      $reccurrence = TRUE;
      if ($reccurrence){
          update_option('Statjus_zal','lll');
        //====================RECCURENCE
        //coupon on reccurence
        if ($coupon_data){
          if (!empty($coupon_data['reccuring'])){
            //everytime the price will be reduced
            $level_arr['price'] = ihc_coupon_return_price_after_decrease($level_arr['price'], $coupon_data, TRUE, $this->attributes['uid'], $this->attributes['lid']);
            if (isset($level_arr['access_trial_price'])){
              $level_arr['access_trial_price'] = ihc_coupon_return_price_after_decrease($level_arr['access_trial_price'], $coupon_data, FALSE);
            }
          } else {
            //only one time
            if (!empty($level_arr['access_trial_price'])){
              $level_arr['access_trial_price'] = ihc_coupon_return_price_after_decrease($level_arr['access_trial_price'], $coupon_data, TRUE, $uid, $this->attributes['lid']);
            } else {
              $level_arr['access_trial_price'] = ihc_coupon_return_price_after_decrease($level_arr['price'], $coupon_data, TRUE, $uid, $this->attributes['lid']);
              $level_arr['access_trial_type'] = 2;
            }
            if (empty($level_arr['access_trial_type'])){
              $level_arr['access_trial_type'] = 2;
            }
          }
        }

        //trial block
        
        //end of trial

        /// TAXES
        //$levelData['price'] = $this->addTaxes($levelData['price']);

        //coupon
        if ($coupon_data){
          $level_arr['price'] = ihc_coupon_return_price_after_decrease($level_arr['price'], $coupon_data, TRUE, $uid, $this->attributes['lid']);
        }

        /// TAXES
        //$levelData['price'] = $this->addTaxes($levelData['price']);
        /*
        $this->redirectUrl .= '?amount=' . urlencode($level_arr['price']) . '&';
        \Ihc_User_Logs::write_log( __('PayPal Payment: amount set @ ', 'ihc') . $level_arr['price'] . $this->currency, 'payments');
        $this->redirectUrl .= 'paymentaction=sale&';
        */
      //}
      
      if (ihc_is_magic_feat_active('level_dynamic_price') && isset($this->attributes['ihc_dynamic_price'])){
              $temp_amount = $this->attributes['ihc_dynamic_price'];
              if (ihc_check_dynamic_price_from_user($this->attributes['lid'], $temp_amount)){
                  $amount = $temp_amount;
                  \Ihc_User_Logs::write_log( $this->paymentTypeLabel . __(': Dynamic price on - Amount is set by the user @ ', 'ihc') . $amount . $this->currency, 'payments');
              }
       }
          /**************************** DYNAMIC PRICE ***************************/

        //  $amount = number_format( (float)$amount, 2, '.', '' );
       
      $transactionData = array(
        'lid'                 => $this->attributes['lid'],
        'uid'                 => $this->attributes['uid'],
        'ihc_payment_type'    => 'PayIr',
        'amount'              => $level_arr['price'],
        'message'             => 'pending',
        'currency'            => $this->currency,
        'item_name'           => $level_arr['name'],
      );
	  global $current_user;

      
/////////////////////

	  $paymentId = time() . $current_user->ID;
	  $api = $payir_email;
      $amount = urlencode($level_arr['price']);
      $mobile = null;
      $factorNumber = null;
      $description = $level_arr['name'];
      $redirect = $site_url;
      $result = send_payir($api, $amount, $redirect, $mobile, $factorNumber, $description);
      $result = json_decode($result);
	  if($result->status) {		  
		 $go = "https://pay.ir/pg/$result->token";
		 $this->redirectUrl = $go;
		 update_option( 'PayIr'.$this->attributes['uid'], 'PayIr' );
         update_option( 'price'.$this->attributes['uid'], $level_arr['price'] );
         update_option( 'lid'.$this->attributes['uid'], $this->attributes['lid'] );
         $r_url = get_option('ihc_PayIr_return_page');
         update_option( 'r_url'.$this->attributes['uid'], $r_url );
	     ihc_insert_update_transaction( $this->attributes['uid'], $paymentId, $transactionData, true ); /// will save the order too

         /// update indeed_members_payments table, add order id
         \Ihc_Db::updateTransactionAddOrderId( $paymentId, @$this->attributes['orderId'] );
      } else {
	     echo $result->errorMessage;
      }  
      
      return $this;
      }
	  return $this;
    }

  	public function redirect()
    {
        \Ihc_User_Logs::write_log( __('PayPal Payment: Request submited.', 'ihc'), 'payments');
        header( 'location:' . $this->redirectUrl);
        exit();
    }

    public function webhook(){
      ini_set('display_errors','on');


      if (get_option('ihc_debug_payments_db')){
        ihc_insert_debug_payment_log('PayIr', $_POST);
      }

      \Ihc_User_Logs::write_log( __('PayPal Payment IPN: Start process', 'ihc'), 'payments');
      $PayIr_email = get_option('ihc_PayIr_email');        
      $price = get_option('price'.get_current_user_id());
		
	  $token = $_GET['token'];
      $result = json_decode(verify_payir($payir_email,$token));
      if(isset($result->status)){
	     if($result->status == 1){
			$Status = 'completed';
            $Transaction_ID = $result['RefID'];
       
			$lid = get_option('lid'.get_current_user_id());
	    	$level_data = ihc_get_level_by_id($lid);
            update_option( 'lid', $lid );
			ihc_set_level_trial_time_for_no_pay($level_data, $lid);
            \Ihc_User_Logs::write_log( __("PayIr Payment IPN: Update user level expire time (Trial).", 'ihc'), 'payments');
            ihc_send_user_notifications($lid, 'payment', $lid);//send notification to user
            ihc_send_user_notifications($lid, 'admin_user_payment', $lid);//send notification to admin
            do_action( 'ihc_payment_completed', $lid, $lid );
            ihc_switch_role_for_user($lid);
            //ihc_insert_update_transaction($lid, $insert_data['txn_id'], $insert_data);
		
		    ///////////		
		    Ihc_User_Logs::write_log( __('PayIr Payment Is Verified. Refrence code is: ', 'ihc') . $Authority, 'payments');
		    Ihc_User_Logs::set_user_id(get_current_user_id());
		    Ihc_User_Logs::set_level_id($lid);
		    Ihc_User_Logs::write_log( __("PayIr Payment IPN: set user id @ ", 'ihc') . get_current_user_id(), 'payments');
		    Ihc_User_Logs::write_log( __("PayIr Payment IPN: Update user level expire time.", 'ihc'), 'payments');
		    ihc_update_user_level_expire($level_data, $lid, get_current_user_id());						
		    ihc_send_user_notifications(get_current_user_id(), 'payment', $lid);//send notification to user
		    ihc_send_user_notifications(get_current_user_id(), 'admin_user_payment', $lid);//send notification to admin
		    ihc_switch_role_for_user(get_current_user_id());
		
		    ihc_set_level_trial_time_for_no_pay($lid, get_current_user_id());		
		    ihc_send_user_notifications(get_current_user_id(), 'payment', $lid);		
		    ihc_send_user_notifications(get_current_user_id(), 'admin_user_payment', $lid);//send notification to admin
		
		    do_action( 'ihc_payment_completed', get_current_user_id(), $lid );
		    ihc_switch_role_for_user(get_current_user_id());
			//////////
		    ///////
            $debug = FALSE;
            $path = str_replace('PayIr_ipn.php', '', __FILE__);
            $log_file = $path . 'PayIr.log';
            $raw_post_data = file_get_contents('php://input');
            \Ihc_User_Logs::write_log( __('PayIr Payment IPN: Extract data from response.', 'ihc'), 'payments');
            $raw_post_array = explode('&', $raw_post_data);
            $myPost = array();
            foreach ($raw_post_array as $keyval) {
              $keyval = explode ('=', $keyval);
              if (count($keyval) == 2)
                $myPost[$keyval[0]] = urldecode($keyval[1]);
            }
            // read the post from PayPal system and add 'cmd'
            $req = 'cmd=_notify-validate';
            if (function_exists('get_magic_quotes_gpc')) {
              $get_magic_quotes_exists = true;
            }
            foreach ($myPost as $key => $value) {
              if ($get_magic_quotes_exists == true && get_magic_quotes_gpc() == 1) {
                $value = urlencode(stripslashes($value));
              } else {
                $value = urlencode($value);
              }
              $req .= "&$key=$value";
            }        
            exit(); 
		 }
	  }else {
	      if($_GET['status'] == 0){
		     echo '============= Ultimate Membership Pro - PAYPAL IPN ============= ';
             echo '<br/><br/>No Payments details sent. Come later';
             exit();
	      }
	  }
	  
    }
	
    public function cancelSubscription( $dont_redirect_paypal=false )
    {
        if (!empty($dont_redirect_paypal)){
          return;
        }
        $sandbox = get_option('ihc_PayIr_sandbox');
        $alias = get_option('ihc_PayIr_email');
        $merchant_id = get_option('ihc_PayIr_merchant_account_id');
        if ($merchant_id){
          $alias = $merchant_id;
        }
        if ($sandbox){
          $url = "https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_subscr-find&alias=" . urlencode($alias);
        } else {
          $url = "https://www.paypal.com/cgi-bin/webscr?cmd=_subscr-find&alias=" . urlencode($alias);
        }
        wp_redirect($url);
        exit();
    }

}
