<?php
namespace Indeed\Ihc\PaymentGateways;
/*
@since 7.4
*/


class ZarinPal extends \Indeed\Ihc\PaymentGateways\PaymentAbstract
{

  protected $attributes       = array();
  protected $redirectUrl      = '';
  protected $abort            = false;
  protected $paymentTypeLabel = 'ZarinPal Payment';
  protected $currency         = '';
 
    public function __construct()
    {
        $this->currency = get_option('ihc_currency');
        //include_once IHC_PATH . 'classes/PaymentGateways/zarinpal/lib/nusoap.php';
    }

    public function SendRequestToZarinPal($action, $params){
        try {
            $ch = curl_init('https://www.zarinpal.com/pg/rest/WebGate/' . $action . '.json');
            curl_setopt($ch, CURLOPT_USERAGENT, 'ZarinPal Rest Api v1');
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
               'Content-Type: application/json',
               'Content-Length: ' . strlen($params)
            ));
            $result = curl_exec($ch);
            return json_decode($result, true);
           
        } catch (Exception $ex) {
            return false;
        }
        
    }
  	public function doPayment(){
      \Ihc_User_Logs::set_user_id(@$this->attributes['uid']);
      \Ihc_User_Logs::set_level_id(@$this->attributes['lid']);
      \Ihc_User_Logs::write_log( __('zarinpal Payment: Start process', 'ihc'), 'payments');
      //$client = new SoapClient('https://www.zarinpal.com/pg/services/WebGate/wsdl', ['encoding' => 'UTF-8']);
      $zarinpal_email = get_option('ihc_zarinpal_email');
      $levels = get_option('ihc_levels');
      $sandbox = get_option('ihc_zarinpal_sandbox');
      $r_url = get_option('ihc_zarinpal_return_page');

      if(!$r_url || $r_url==-1){
        $r_url = get_option('page_on_front');
      }
      $r_url = get_permalink($r_url);
      if (!$r_url){
        $r_url = get_home_url();
      }
      
      $err = false;
      if (isset($levels[$this->attributes['lid']])){
        $level_arr = $levels[$this->attributes['lid']];
        if ($level_arr['payment_type']=='free' || $level_arr['price']=='') $err = true;
      } else {
        \Ihc_User_Logs::write_log( __('zarinpal Payment: Level is free, no payment required.', 'ihc'), 'payments');
        $err = true;
      }
      // USER ID
      if (!empty($this->attributes['uid'])){
        $uid = $this->attributes['uid'];
      } else {
        $uid = get_current_user_id();
      }
      if (!$uid){
        $err = true;
        \Ihc_User_Logs::write_log( __('zarinpal Payment: Error, user id not set.', 'ihc'), 'payments');
      } else {
        \Ihc_User_Logs::write_log( __('zarinpal Payment: set user id @ ', 'ihc') . $uid, 'payments');
      }

      /// dynamic price
      $level_arr['price'] = $this->applyDynamicPrice($level_arr['price']);

      if ($err){
        ////if level it's not available for some reason, go back to prev page
        \Ihc_User_Logs::write_log( __('zarinpal Payment: stop process, redirect home.', 'ihc'), 'payments');
        //header( 'location:'. $r_url );
        //exit();
      } else {
        $custom_data = json_encode(array('user_id' => $uid, 'level_id' => $this->attributes['lid'] ));
      }

      $site_url = site_url();
      $site_url = trailingslashit($site_url);
      $notify_url = add_query_arg('ihc_action', 'zarinpal', $site_url);

      \Ihc_User_Logs::write_log( __('zarinpal Payment: set ipn url @ ', 'ihc') . $notify_url, 'payments');

      $reccurrence = FALSE;
      if (isset($level_arr['access_type']) && $level_arr['access_type']=='regular_period'){
        $reccurrence = TRUE;
        \Ihc_User_Logs::write_log( __('zarinpal Payment: Recurrence payment set.', 'ihc'), 'payments');
      }

      //coupons
      $coupon_data = array();
      if (!empty($this->attributes['ihc_coupon'])){
        $coupon_data = ihc_check_coupon($this->attributes['ihc_coupon'], $this->attributes['lid'] );
        \Ihc_User_Logs::write_log( __('PayPal Payment: the user used the following coupon: ', 'ihc') . $this->attributes['ihc_coupon'], 'payments');
      }
      
      update_option('Status_zal','lll');
      $reccurrence = TRUE;
      if ($reccurrence){
          update_option('Statjus_zal','lll');
        //====================RECCURENCE
        //coupon on reccurence
        if ($coupon_data){
          if (!empty($coupon_data['reccuring'])){
            //everytime the price will be reduced
            $level_arr['price'] = ihc_coupon_return_price_after_decrease($level_arr['price'], $coupon_data, TRUE, $this->attributes['uid'], $this->attributes['lid']);
            if (isset($level_arr['access_trial_price'])){
              $level_arr['access_trial_price'] = ihc_coupon_return_price_after_decrease($level_arr['access_trial_price'], $coupon_data, FALSE);
            }
          } else {
            //only one time
            if (!empty($level_arr['access_trial_price'])){
              $level_arr['access_trial_price'] = ihc_coupon_return_price_after_decrease($level_arr['access_trial_price'], $coupon_data, TRUE, $uid, $this->attributes['lid']);
            } else {
              $level_arr['access_trial_price'] = ihc_coupon_return_price_after_decrease($level_arr['price'], $coupon_data, TRUE, $uid, $this->attributes['lid']);
              $level_arr['access_trial_type'] = 2;
            }
            if (empty($level_arr['access_trial_type'])){
              $level_arr['access_trial_type'] = 2;
            }
          }
        }

        //trial block
        
        //end of trial

        /// TAXES
        //$levelData['price'] = $this->addTaxes($levelData['price']);

        //coupon
        if ($coupon_data){
          $level_arr['price'] = ihc_coupon_return_price_after_decrease($level_arr['price'], $coupon_data, TRUE, $uid, $this->attributes['lid']);
        }

        /// TAXES
        //$levelData['price'] = $this->addTaxes($levelData['price']);
        /*
        $this->redirectUrl .= '?amount=' . urlencode($level_arr['price']) . '&';
        \Ihc_User_Logs::write_log( __('PayPal Payment: amount set @ ', 'ihc') . $level_arr['price'] . $this->currency, 'payments');
        $this->redirectUrl .= 'paymentaction=sale&';
        */
      //}
      
      if (ihc_is_magic_feat_active('level_dynamic_price') && isset($this->attributes['ihc_dynamic_price'])){
              $temp_amount = $this->attributes['ihc_dynamic_price'];
              if (ihc_check_dynamic_price_from_user($this->attributes['lid'], $temp_amount)){
                  $amount = $temp_amount;
                  \Ihc_User_Logs::write_log( $this->paymentTypeLabel . __(': Dynamic price on - Amount is set by the user @ ', 'ihc') . $amount . $this->currency, 'payments');
              }
       }
          /**************************** DYNAMIC PRICE ***************************/

        //  $amount = number_format( (float)$amount, 2, '.', '' );
       
      $transactionData = array(
        'lid'                 => $this->attributes['lid'],
        'uid'                 => $this->attributes['uid'],
        'ihc_payment_type'    => 'zarinpal',
        'amount'              => $level_arr['price'],
        'message'             => 'pending',
        'currency'            => $this->currency,
        'item_name'           => $level_arr['name'],
      );
	  global $current_user;

      
/////////////////////

	  $paymentId = time() . $current_user->ID;
	  $data = array('MerchantID' => $zarinpal_email, 'Amount' => $level_arr['price'], 'CallbackURL' => $site_url, 'Description' => $level_arr['name']);

      update_option('Status_zarinpal1oo','1');
      $result = $this->SendRequestToZarinPal('PaymentRequest', json_encode($data));
      if ($result === false) {
          update_option('Status_zarinpal','2');
          echo "cURL Error #:" . $err;
      } else {
        if ($result["Status"] == 100) {
            update_option('Status_zarinpal','1');
            //wp_redirect(sprintf($acczarin, $result['Authority']));
            //Header('Location: https://www.zarinpal.com/pg/StartPay/'.$result['Authority']);
            //Header('Location: https://www.zarinpal.com/pg/StartPay/' . $result["Authority"]);
		    $this->redirectUrl = 'https://www.zarinpal.com/pg/StartPay/' . $result["Authority"];
		    update_option( 'zarinpal'.$this->attributes['uid'], 'zarinpal' );
            update_option( 'price'.$this->attributes['uid'], $level_arr['price'] );
            update_option( 'lid'.$this->attributes['uid'], $this->attributes['lid'] );
            $r_url = get_option('ihc_zarinpal_return_page');
            update_option( 'r_url'.$this->attributes['uid'], $r_url );
	        ihc_insert_update_transaction( $this->attributes['uid'], $paymentId, $transactionData, true ); /// will save the order too

            /// update indeed_members_payments table, add order id

            \Ihc_Db::updateTransactionAddOrderId( $paymentId, @$this->attributes['orderId'] );
            //return $this;
        } else {
            update_option('Status_zarinpal','5');
            $Message = ' تراکنش ناموفق بود- کد خطا : ' . $result["Status"];
            $Fault = '';
        }
      }
      
      
      return $this;
      }
	  return $this;
    }

  	public function redirect()
    {
        \Ihc_User_Logs::write_log( __('PayPal Payment: Request submited.', 'ihc'), 'payments');
        header( 'location:' . $this->redirectUrl);
        exit();
    }

    public function webhook()
    {
      ini_set('display_errors','on');


      if (get_option('ihc_debug_payments_db')){
        ihc_insert_debug_payment_log('zarinpal', $_POST);
      }

      \Ihc_User_Logs::write_log( __('PayPal Payment IPN: Start process', 'ihc'), 'payments');
      
      if ($_GET['Status'] == 'OK') {
		$Authority = $_GET['Authority'];
	    $zarinpal_email = get_option('ihc_zarinpal_email');
        
        $price = get_option('price'.get_current_user_id());
        
        $Authority = $_GET['Authority'];
        $data = array('MerchantID' => $zarinpal_email, 'Authority' => $Authority, 'Amount' => $Amount);
        
        $result = $this->SendRequestToZarinPal('PaymentVerification', json_encode($data));

        if ($result['Status'] == 100) {
            $Status = 'completed';
            $Transaction_ID = $result['RefID'];
       
			$lid = get_option('lid'.get_current_user_id());
	    	$level_data = ihc_get_level_by_id($lid);
            update_option( 'lid', $lid );
			ihc_set_level_trial_time_for_no_pay($level_data, $lid);
            \Ihc_User_Logs::write_log( __("zarinpal Payment IPN: Update user level expire time (Trial).", 'ihc'), 'payments');
            ihc_send_user_notifications($lid, 'payment', $lid);//send notification to user
            ihc_send_user_notifications($lid, 'admin_user_payment', $lid);//send notification to admin
            do_action( 'ihc_payment_completed', $lid, $lid );
            ihc_switch_role_for_user($lid);
            //ihc_insert_update_transaction($lid, $insert_data['txn_id'], $insert_data);
		
		    ///////////		
		Ihc_User_Logs::write_log( __('ZarinPal Payment Is Verified. Refrence code is: ', 'ihc') . $Authority, 'payments');
		Ihc_User_Logs::set_user_id(get_current_user_id());
		Ihc_User_Logs::set_level_id($lid);
		Ihc_User_Logs::write_log( __("ZarinPal Payment IPN: set user id @ ", 'ihc') . get_current_user_id(), 'payments');
		Ihc_User_Logs::write_log( __("ZarinPal Payment IPN: Update user level expire time.", 'ihc'), 'payments');
		ihc_update_user_level_expire($level_data, $lid, get_current_user_id());						
		ihc_send_user_notifications(get_current_user_id(), 'payment', $lid);//send notification to user
		ihc_send_user_notifications(get_current_user_id(), 'admin_user_payment', $lid);//send notification to admin
		ihc_switch_role_for_user(get_current_user_id());
		
		ihc_set_level_trial_time_for_no_pay($lid, get_current_user_id());		
		ihc_send_user_notifications(get_current_user_id(), 'payment', $lid);		
		ihc_send_user_notifications(get_current_user_id(), 'admin_user_payment', $lid);//send notification to admin
		
		do_action( 'ihc_payment_completed', get_current_user_id(), $lid );
		ihc_switch_role_for_user(get_current_user_id());
			//////////
		///////
        $debug = FALSE;
        $path = str_replace('zarinpal_ipn.php', '', __FILE__);
        $log_file = $path . 'zarinpal.log';
        $raw_post_data = file_get_contents('php://input');
        \Ihc_User_Logs::write_log( __('zarinpal Payment IPN: Extract data from response.', 'ihc'), 'payments');
        $raw_post_array = explode('&', $raw_post_data);
        $myPost = array();
        foreach ($raw_post_array as $keyval) {
          $keyval = explode ('=', $keyval);
          if (count($keyval) == 2)
            $myPost[$keyval[0]] = urldecode($keyval[1]);
        }
        // read the post from PayPal system and add 'cmd'
        $req = 'cmd=_notify-validate';
        if (function_exists('get_magic_quotes_gpc')) {
          $get_magic_quotes_exists = true;
        }
        foreach ($myPost as $key => $value) {
          if ($get_magic_quotes_exists == true && get_magic_quotes_gpc() == 1) {
            $value = urlencode(stripslashes($value));
          } else {
            $value = urlencode($value);
          }
          $req .= "&$key=$value";
        }
        
        
	  
      exit();
    }
      } else {
        echo '============= Ultimate Membership Pro - PAYPAL IPN ============= ';
        echo '<br/><br/>No Payments details sent. Come later';
        exit();
      }
    }
    public function cancelSubscription( $dont_redirect_paypal=false )
    {
        if (!empty($dont_redirect_paypal)){
          return;
        }
        $sandbox = get_option('ihc_zarinpal_sandbox');
        $alias = get_option('ihc_zarinpal_email');
        $merchant_id = get_option('ihc_zarinpal_merchant_account_id');
        if ($merchant_id){
          $alias = $merchant_id;
        }
        if ($sandbox){
          $url = "https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_subscr-find&alias=" . urlencode($alias);
        } else {
          $url = "https://www.paypal.com/cgi-bin/webscr?cmd=_subscr-find&alias=" . urlencode($alias);
        }
        wp_redirect($url);
        exit();
    }

}
